import os
import pathlib
import praw
import requests
import shutil
from imgurdownloader import ImgurDownloader
from config import CONFIG


def get_subreddits(reddit):
    """Get a list of subreddit names.

    If any multireddits are being searched, this function grabs the name of
    each sub for the list.

    Parameters
    ----------
    reddit : praw.Reddit
        The reddit instance used to gather data.

    Returns
    -------
    subreddits : set
        Set of subreddit names.

    """

    allow_nsfw = CONFIG["Downloads"].getboolean("AllowNSFW")
    subreddit_config = CONFIG.options("Subreddits")
    subreddits = []

    if not allow_nsfw:
        for subreddit in subreddit_config:
            sub = reddit.subreddit(subreddit)
            if not sub.over18:
                subreddits.append(sub.display_name)
    else:
        subreddits = subreddit_config

    for sub in subreddits:
        sub = sub.lower()


    multi_config = CONFIG.options("Multireddits")
    multi_list = [tuple(m.split()) for m in multi_config]

    for user, multi in multi_list:
        multireddit = reddit.multireddit(user, multi)
        if not allow_nsfw and multireddit.over_18:
            continue
        for sub in multireddit.subreddits:
            subreddits.append(sub.display_name.lower())

    subreddits = set(subreddits)
    return subreddits


def download_post_image(post):
    """Download the image associated with the given reddit post."""

    if post.is_self:
        return False

    # FIXME: Check file permissions when attempting to create directory
    output_dir = pathlib.Path(CONFIG["Downloads"]["OutputDirectory"]).expanduser()
    if str(output_dir) == '.':
        output_dir = pathlib.Path.cwd()  # Get the absolute path
    if not output_dir.exists():
        output_dir.mkdir(parents=True)

    image_types = (".png", ".jpg", ".jpeg", ".gif")
    if not post.url.endswith(image_types):
        return False

    filename_start = post.url.rfind('/')
    filename = post.url[filename_start:]

    output_path = str(output_dir) + filename

    with open(output_path, "wb") as handle:
        response = requests.get(post.url, stream=True)
        if not response.ok:
            print(response)
            return False

        for block in response.iter_content(1024):
            if not block:
                break
            handle.write(block)

    return True


def main():
    """Main entry point of the program."""

    reddit = praw.Reddit("reddit-wall")

    subreddits = get_subreddits(reddit)
    subs = reddit.subreddit('+'.join(subreddits))
    limit = int(CONFIG["Downloads"]["PostLimit"])
    allow_nsfw = CONFIG["Downloads"].getboolean("AllowNSFW")

    # There doesn't seem to be a way to specify a nsfw filter for subs.hot(),
    # so we have to keep track of when we skip posts.
    posts = []
    for submission in subs.hot():
        if not allow_nsfw and submission.over_18:
            continue
        
        posts.append(submission)
        if len(posts) > limit:
            break

    imgur = ImgurDownloader()
    for post in posts:
        if "imgur.com/a/" in post.url:
            imgur.download_album(post.url)
        else:
            download_post_image(post)


if __name__ == "__main__":
    main()



import pathlib
import requests
from imgurpython import ImgurClient
from config import CONFIG


class ImgurDownloader:

    def __init__(self):
        client_id = CONFIG["Imgur"]["client_id"]
        client_secret = CONFIG["Imgur"]["client_secret"]
        self.client = ImgurClient(client_id, client_secret)

    
    #FIXME: Implement this
    def check_rate_limit(self):
        pass

    
    def get_imgur_id(self, url):
        """Get an Imgur id from a url.
        
        Returns
        -------
        imgur_id : string
            The id of the content at the url.
        
        """

        if "imgur" not in url:
            return None

        imgur_id = url[url.rfind('/') + 1:]

        extension_index = imgur_id.find('.')
        if extension_index != -1:
            imgur_id = imgur_id[:extension_index]

        return imgur_id


    #FIXME: Add error checking
    def download_album(self, url):
        """Downloads the imgur album from the given url.
        
        Parameters
        ----------
        url : string
            URL for the imgur album.

        output_dir : string
            Path to the directory for saving images.
            
        Returns
        -------
        success : bool
            Whether the album downloaded successfully.

        """

        if "imgur.com/a/" not in url:
            return False

        output_dir = pathlib.Path(CONFIG["Downloads"]["OutputDirectory"]).expanduser()
        if str(output_dir) == '.':
            output_dir = pathlib.Path.cwd()  # Get the absolute path
        if not output_dir.exists():
            output_dir.mkdir(parents=True)

        output_dir_str = str(output_dir)
        if not output_dir_str.endswith('/'):
            output_dir_str = output_dir_str + '/'

        imgur_id = self.get_imgur_id(url)
        album = self.client.get_album(imgur_id)

        for image in album.images:
            if type(image) is dict:
                image_id = image["id"]
                image_type = image["type"]
                image_link = image["link"]
            else:
                image_id = image.id
                image_type = image.type
                image_link = image.link
            
            output_filename = output_dir_str + image_id + " - imgur." + image_type[6:]

            with open(output_filename, 'wb') as handle:
                response = requests.get(image_link, stream=True)
                if not response.ok:
                    print(response)
                    return False
                
                for block in response.iter_content(1024):
                    if not block:
                        break
                    handle.write(block)

        return True


"""Configuration module.

This module provides functions for finding, reading, and creating user
configuration files. It also provides an object, CONFIG, for easy access
to the loaded config.
"""

import os
import shutil
from configparser import ConfigParser

DEFAULT_CONFIG_FILE = "~/.config/reddit-wall.conf"

# FIXME: Add support for specifying config path as a command line argument
def create_config(config_file=None):
    """Read the program config file and create one if it doesn't exist.

    Parameters
    ----------
    config_file : string
        Path to the config file if not using the default.

     Returns
     -------
     parser : ConfigParser
        An instance of the program configuration.

    """

    default_config_path = os.path.expanduser(DEFAULT_CONFIG_FILE)
    if not config_file and not os.path.exists(default_config_path):
        shutil.copyfile("conf/reddit-wall.conf", default_config_path)

    parser = ConfigParser(allow_no_value=True)

    parser.read_file(open(default_config_path))
    if config_file:
        parser.read(config_file)

    return parser

CONFIG = create_config()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="reddit-wall",
    version="0.1.0",
    author="Jalen Adams",
    author_email="jalen@jalenkadams.me",
    description="Download wallpapers from subreddits and multireddits of your choosing.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LeftySolara/reddit-wall",
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": [
            "reddit-wall=main:main"
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Environment :: Console",
        "Natural Language :: English"
    ],
)
